#
# (C) Copyright 2018 InovaDevelopment.com
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# Author: Karl  Schopmeyer <inovadevelopment.com>
#

"""
Utility functionsto support pywbem_mock.
"""

from __future__ import absolute_import, print_function

import locale
import sys
import six

if six.PY2:
    import codecs  # pylint: disable=wrong-import-order

STDOUT_ENCODING = getattr(sys.stdout, 'encoding', None)
if not STDOUT_ENCODING:
    STDOUT_ENCODING = locale.getpreferredencoding()
if not STDOUT_ENCODING:
    STDOUT_ENCODING = 'utf-8'


def _uprint(dest, text):
    """
    Write text to dest, adding a newline character.

    Text may be a unicode string, or a byte string in UTF-8 encoding.
    It must not be None.

    If dest is None, the text is encoded to a codepage suitable for the current
    stdout and is written to stdout.

    Otherwise, dest must be a file path, and the text is encoded to a UTF-8
    Byte sequence and is appended to the file (opening and closing the file).
    """
    if isinstance(text, six.text_type):
        text = text + u'\n'
    elif isinstance(text, six.binary_type):
        text = text + b'\n'
    else:
        raise TypeError(
            "text must be a unicode or byte string, but is {0}".
            format(type(text)))
    if dest is None:
        if six.PY2:
            # On py2, stdout.write() requires byte strings
            if isinstance(text, six.text_type):
                text = text.encode(STDOUT_ENCODING, 'replace')
        else:
            # On py3, stdout.write() requires unicode strings
            if isinstance(text, six.binary_type):
                text = text.decode('utf-8')
        sys.stdout.write(text)
    elif isinstance(dest, (six.text_type, six.binary_type)):
        if isinstance(text, six.text_type):
            open_kwargs = dict(mode='a', encoding='utf-8')
        else:
            open_kwargs = dict(mode='ab')
        if six.PY2:
            # Open with codecs to be able to set text mode
            with codecs.open(dest, **open_kwargs) as f:
                f.write(text)
        else:
            with open(dest, **open_kwargs) as f:
                f.write(text)
    else:
        raise TypeError(
            "dest must be None or a string, but is {0}".
            format(type(text)))



"""
End2end tests for SNIA 'Server' profile.
"""

from __future__ import absolute_import, print_function

import warnings

from .utils.pytest_extensions import ProfileTest
from .utils.assertions import assert_number_of_instances_equal, \
    assert_number_of_instances_minimum, assert_instance_of, \
    assert_instance_consistency, assert_mandatory_properties, \
    assert_property_one_of, assert_property_contains
from .utils.utils import server_func_asserted, server_prop_asserted

# Note: The wbem_connection fixture uses the server_definition fixture, and
# due to the way py.test searches for fixtures, it also need to be imported.
# pylint: disable=unused-import
from .utils.pytest_extensions import wbem_connection  # noqa: F401
from .utils.pytest_extensions import server_definition  # noqa: F401
from .utils.pytest_extensions import assert_association_func  # noqa: F401
# pylint: enable=unused-import

# pylint: disable=wrong-import-position, wrong-import-order, invalid-name
from ..utils import import_installed
pywbem = import_installed('pywbem')
from pywbem import ToleratedServerIssueWarning  # noqa: E402
from pywbem._utils import _format  # noqa: E402
# pylint: enable=wrong-import-position, wrong-import-order, invalid-name


class Test_SNIA_Server_Profile(ProfileTest):
    """
    All end2end tests for SNIA 'Server' profile.
    """

    def init_profile(self, conn):  # pylint: disable=arguments-differ
        super(Test_SNIA_Server_Profile, self).init_profile(
            conn, profile_org='SNIA', profile_name='Server')

    def test_get_central_instances(self, wbem_connection):  # noqa: F811
        # pylint: disable=redefined-outer-name
        """
        Test WBEMServer.get_central_instances() for this profile.
        """
        self.init_profile(wbem_connection)

        central_inst_paths = server_func_asserted(
            self.server, 'get_central_instances',
            profile_path=self.profile_inst.path,
            central_class=self.profile_definition['central_class'],
            scoping_class=self.profile_definition['scoping_class'],
            scoping_path=self.profile_definition['scoping_path'],
            reference_direction=self.profile_definition['reference_direction'])

        central_insts_msg = _format(
            "central instances of profile {0} {1!A} (version: {2})",
            self.profile_definition['registered_org'],
            self.profile_definition['registered_name'],
            self.profile_definition['registered_version'] or 'any')

        # Check that there is just one central instance for this profile
        assert_number_of_instances_equal(
            wbem_connection, central_inst_paths, central_insts_msg, 1)

    def test_central_instance(self, wbem_connection):  # noqa: F811
        # pylint: disable=redefined-outer-name
        """
        Test the CIM_ObjectManager central instance.
        """
        self.init_profile(wbem_connection)
        self.init_central_instances()
        central_inst_path = self.central_inst_paths[0]

        assert_instance_of(
            wbem_connection, central_inst_path, 'CIM_ObjectManager')

        central_inst = self.conn.GetInstance(
            central_inst_path,
            asserted=True)

        assert_instance_of(
            wbem_connection, central_inst, 'CIM_ObjectManager')

        assert_instance_consistency(
            wbem_connection, central_inst, central_inst_path)

        assert_mandatory_properties(
            wbem_connection, central_inst,
            ['SystemCreationClassName', 'SystemName', 'CreationClassName',
             'Name', 'ElementName', 'Description', 'OperationalStatus',
             'Started'])

    def test_Namespace(
            self, assert_association_func, wbem_connection):  # noqa: F811
        # pylint: disable=redefined-outer-name
        """
        Test the associated CIM_Namespace instances.
        """
        self.init_profile(wbem_connection)
        self.init_central_instances()
        central_inst_path = self.central_inst_paths[0]

        far_insts, _ = assert_association_func(
            conn=wbem_connection,
            profile_id=self.profile_id,
            source_path=central_inst_path,
            source_role='Antecedent',
            assoc_class='CIM_NamespaceInManager',
            far_role='Dependent',
            far_class='CIM_Namespace')

        far_insts_msg = _format(
            "CIM_Namespace instances associated via "
            "CIM_NamespaceInManager to "
            "central instance of profile {0} {1!A} (version {2})",
            self.profile_definition['registered_org'],
            self.profile_definition['registered_name'],
            self.profile_definition['registered_version'] or 'any')

        for inst in far_insts:

            assert_mandatory_properties(
                wbem_connection, inst,
                ['SystemCreationClassName', 'SystemName',
                 'ObjectManagerCreationClassName', 'ObjectManagerName',
                 'CreationClassName', 'Name', 'ClassType'])

        # Check that they are the namespaces determined by WBEMServer.
        # Because the Interop namespace is added if missing, we need to also
        # do that here.
        inst_names = [inst['Name'].lower() for inst in far_insts]
        interop_ns_lower = server_prop_asserted(
            self.server, 'interop_ns').lower()
        if interop_ns_lower not in inst_names:
            inst_names.append(interop_ns_lower)
        determined_names = [ns.lower()
                            for ns in server_prop_asserted(
                                self.server, 'namespaces')]

        if set(inst_names) != set(determined_names):
            raise AssertionError(
                _format("Server {0} at {1}: The namespaces ({2}) of the {3} "
                        "do not match the namespaces ({4}) determined by the "
                        "WBEMServer class",
                        wbem_connection.server_definition.nickname,
                        wbem_connection.url,
                        set(inst_names), far_insts_msg,
                        set(determined_names)))

    def test_ObjectManagerCommunicationMechanism(
            self, assert_association_func, wbem_connection):  # noqa: F811
        # pylint: disable=redefined-outer-name
        """
        Test the associated CIM_ObjectManagerCommunicationMechanism instances.
        """
        self.init_profile(wbem_connection)
        self.init_central_instances()
        central_inst_path = self.central_inst_paths[0]

        far_insts, _ = assert_association_func(
            conn=wbem_connection,
            profile_id=self.profile_id,
            source_path=central_inst_path,
            source_role='Antecedent',
            assoc_class='CIM_CommMechanismForManager',
            far_role='Dependent',
            far_class='CIM_ObjectManagerCommunicationMechanism')

        far_insts_msg = _format(
            "CIM_ObjectManagerCommunicationMechanism instances associated via "
            "CIM_CommMechanismForManager to "
            "central instance of profile {0} {1!A} (version {2})",
            self.profile_definition['registered_org'],
            self.profile_definition['registered_name'],
            self.profile_definition['registered_version'] or 'any')

        for inst in far_insts:

            assert_mandatory_properties(
                wbem_connection, inst,
                ['SystemCreationClassName', 'SystemName', 'CreationClassName',
                 'Name', 'ElementName', 'CommunicationMechanism',
                 'FunctionalProfilesSupported', 'MultipleOperationsSupported',
                 'AuthenticationMechanismsSupported'])

            assert_property_one_of(
                wbem_connection, inst,
                'CommunicationMechanism', [2, 4])

            # Note: The SNIA Server profile mandates that the
            #       FunctionalProfilesSupported property is 0 (Unknown), but it
            #       is an array property. Even after fixing this to an array
            #       with one item 0, it does not make much sense to require
            #       that that support is not implemented just because the
            #       profile does not use it.
            # TODO 2018-12 AM: This should be brought back to SMI-S.
            try:
                assert_property_contains(
                    wbem_connection, inst,
                    'FunctionalProfilesSupported', 0)
            except AssertionError as exc:
                warnings.warn("Downgraded the following test failure to a "
                              "warning: {0}: {1}".
                              format(exc.__class__.__name__, exc),
                              ToleratedServerIssueWarning)

            # Note: The SNIA Server profile mandates that the
            #       MultipleOperationsSupported property is False, but it does
            #       not make much sense to require that that support is not
            #       implemented just because the profile does not use it.
            # TODO 2018-12 AM: This should be brought back to SMI-S.
            try:
                assert_property_one_of(
                    wbem_connection, inst,
                    'MultipleOperationsSupported', [False])
            except AssertionError as exc:
                warnings.warn("Downgraded the following test failure to a "
                              "warning: {0}: {1}".
                              format(exc.__class__.__name__, exc),
                              ToleratedServerIssueWarning)

        # Check that there is at least one associated istance
        assert_number_of_instances_minimum(
            wbem_connection, far_insts, far_insts_msg, 1)

    def test_System(
            self, assert_association_func, wbem_connection):  # noqa: F811
        # pylint: disable=redefined-outer-name
        """
        Test the associated CIM_System instance.
        """
        self.init_profile(wbem_connection)
        self.init_central_instances()
        central_inst_path = self.central_inst_paths[0]

        far_insts, _ = assert_association_func(
            conn=wbem_connection,
            profile_id=self.profile_id,
            source_path=central_inst_path,
            source_role='Dependent',
            assoc_class='CIM_HostedService',
            far_role='Antecedent',
            far_class='CIM_System')

        far_insts_msg = _format(
            "CIM_System instances associated via CIM_HostedService to "
            "central instance of profile {0} {1!A} (version {2})",
            self.profile_definition['registered_org'],
            self.profile_definition['registered_name'],
            self.profile_definition['registered_version'] or 'any')

        for inst in far_insts:

            assert_mandatory_properties(
                wbem_connection, inst,
                ['CreationClassName', 'Name', 'NameFormat', 'Description',
                 'ElementName', 'OperationalStatus'])

            assert_property_one_of(
                wbem_connection, inst,
                'NameFormat', ['IP', 'WWN', 'Other'])

        # Check that there is one associated CIM_System instance
        assert_number_of_instances_equal(
            wbem_connection, far_insts, far_insts_msg, 1)



# -*- coding: utf-8 -*-
#
# (C) Copyright 2020 InovaDevelopment.com
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# Author: Karl  Schopmeyer <inovadevelopment.com>
#

"""
Test of of the installation and operation of the CIM_Namespace provider.

"""
from __future__ import absolute_import, print_function

import os
import six
import pytest

from ..utils.pytest_extensions import simplified_test_function
from ..utils.dmtf_mof_schema_def import DMTF_TEST_SCHEMA_VER
from ...utils import skip_if_moftab_regenerated

# pylint: disable=wrong-import-position, wrong-import-order, invalid-name
from ...utils import import_installed
pywbem = import_installed('pywbem')
from pywbem import CIMError, WBEMServer  # noqa: E402
pywbem_mock = import_installed('pywbem_mock')
from pywbem_mock import FakedWBEMConnection, DMTFCIMSchema  # noqa: E402
# pylint: enable=wrong-import-position, wrong-import-order, invalid-name

# Location of DMTF schema directory used by all tests.
# This directory is permanent and should not be removed.
TESTSUITE_SCHEMA_DIR = os.path.join('tests', 'schema')

VERBOSE = False

# test variables to allow selectively executing tests.
OK = True
RUN = True
FAIL = False


def test_interop_namespace_names():
    """
    Test the method interop_namespace_names. This is a single test
    with no parameters because it is really a static operation.
    """
    conn = FakedWBEMConnection()

    interop_ns = conn.interop_namespace_names

    # get valid set from WBEMServer.
    interop_namespaces = WBEMServer.INTEROP_NAMESPACES

    assert set(interop_ns) == set(interop_namespaces)
    # confirm case interop_ns is case-insensitive iterable
    for name in interop_namespaces:
        assert name.lower() in [ns.lower() for ns in interop_ns]


TESTCASES_IS_INTEROP_NAMESPACE = [
    ("verify interop True",
     dict(ns='interop', exp_rtn=True), None, None, OK),

    ("verify root/interop True",
     dict(ns='root/interop', exp_rtn=True), None, None, OK),

    ("verify root/interop True",
     dict(ns='root/pg_interop', exp_rtn=True), None, None, OK),

    ("verify root/blah False",
     dict(ns='root/blah', exp_rtn=False), None, None, OK),
]


@pytest.mark.parametrize(
    "desc, kwargs, exp_exc_types, exp_warn_types, condition",
    TESTCASES_IS_INTEROP_NAMESPACE)
@simplified_test_function
def test_is_interop_namespace(testcase, ns, exp_rtn):
    # pylint: disable=unused-argument
    """
    Test the method _is_interop_namespace() which should return True
    or False depending on whether the tst_ns is a valid interop namespace
    name returns correct result.
    """
    conn = FakedWBEMConnection()

    assert conn.is_interop_namespace(ns) is exp_rtn
    assert conn.is_interop_namespace(ns.upper()) is exp_rtn


TESTCASES_FIND_INTEROP_NAMESPACE = [
    # deflt - default-namespace, string or None
    # nss = Other namespaces to add (String or list of strings)
    # exp_ns = Expected namespace returned

    ("Verify interop not in enviroment returns None",
     dict(deflt=None, nss=None, exp_ns=None), None, None, OK),

    ("Verify interop in enviroment returns interop",
     dict(deflt=None, nss=['interop'], exp_ns='interop'), None, None, OK),

    ("Verify INTEROP in enviroment returns INTEROP",
     dict(deflt=None, nss=['INTEROP'], exp_ns='INTEROP'), None, None, OK),

    ("Verify root/interop in enviroment returns interop",
     dict(deflt=None, nss=['root/interop'], exp_ns='root/interop'), None,
     None, OK),

    ("Verify interop in enviroment with other ns returns interop",
     dict(deflt=None, nss=['interop', 'root/blah'], exp_ns='interop'), None,
     None, OK),

    ("Verify interop in enviroment with other ns and default returns interop",
     dict(deflt='root/cimv2', nss=['interop', 'root/blah'], exp_ns='interop'),
     None, None, OK),

    ("Verify interop in enviroment with other ns and default returns interop",
     dict(deflt='root/cimv2', nss=['root/blah', 'interop'], exp_ns='interop'),
     None, None, OK),

    ("Verify interop as default can be found",
     dict(deflt='interop', nss=None, exp_ns='interop'), None, None, OK),

    ("Verify interop not in namespaces that has a namespace finds nothing",
     dict(deflt=None, nss=['root/blah'], exp_ns=None), None, None, OK),
]


@pytest.mark.parametrize(
    "desc, kwargs, exp_exc_types, exp_warn_types, condition",
    TESTCASES_FIND_INTEROP_NAMESPACE)
@simplified_test_function
def test_find_interop_namespace(testcase, deflt, nss, exp_ns):
    # pylint: disable=unused-argument
    """
    Test the method
    """
    conn = FakedWBEMConnection(default_namespace=deflt)

    if nss:
        if isinstance(nss, six.string_types):
            nss = [nss]
        for name in nss:
            conn.add_namespace(name)

    rtnd_ns = conn.find_interop_namespace()

    assert rtnd_ns == exp_ns


TESTCASES_CIMNAMESPACE_PROVIDER = [

    # Testcases for dictionary tests

    # Each list item is a testcase tuple with these items:
    # * desc: Short testcase description.
    # * kwargs: Keyword arguments for the test function:
    #   * obj: Object that has dictionary behavior, e.g. IMInstanceName.
    #   * exp_dict: Expected dictionary items, for validation.
    # * exp_exc_types: Expected exception type(s), or None.
    # * exp_warn_types: Expected warning type(s), or None.
    # * condition: Boolean condition for testcase to run, or 'pdb' for debugger

    (
        "Test with interop as interop namespace name",
        dict(
            default_ns=None,
            ns='interop',
        ),
        None, None, OK
    ),
    (
        "Test with interop as interop namespace name and root/blah default",
        dict(
            default_ns='root/blah',
            ns='interop',
        ),
        None, None, OK
    ),
    (
        "Test with root/interop as interop namespace name",
        dict(
            default_ns=None,
            ns='root/interop',
        ),
        None, None, OK
    ),
    (
        "Test with invalid interop namespace name",
        dict(
            default_ns=None,
            ns='root/interopx',
        ),
        CIMError, None, OK
    ),
]


@pytest.mark.parametrize(
    "desc, kwargs, exp_exc_types, exp_warn_types, condition",
    TESTCASES_CIMNAMESPACE_PROVIDER)
@simplified_test_function
def test_install_namespace_provider(testcase, default_ns, ns):
    """
    Test installation of the namespace provider
    """
    # setup the connection and schema
    conn = FakedWBEMConnection(default_namespace=default_ns)

    skip_if_moftab_regenerated()

    schema = DMTFCIMSchema(DMTF_TEST_SCHEMA_VER, TESTSUITE_SCHEMA_DIR,
                           verbose=False)
    # code to be tested
    conn.install_namespace_provider(ns, schema.schema_pragma_file)

    # Ensure that exceptions raised in the remainder of this function
    # are not mistaken as expected exceptions
    assert testcase.exp_exc_types is None

    # Assert that the defined interop namespace is installed
    assert ns in conn.namespaces

    if default_ns:
        assert default_ns in conn.namespaces

    assert len(conn.EnumerateInstances("CIM_Namespace", ns)) == \
        len(conn.namespaces)

    for namespace in conn.namespaces:
        if namespace != ns:
            assert len(conn.EnumerateClasses(namespace=namespace)) == 0

# TODO. Test add and remove namespaces with more variations



"""
Test memory leaks for _statistics.py module.
"""

from __future__ import absolute_import, print_function

import pytest
import yagot

# pylint: disable=wrong-import-position, wrong-import-order, invalid-name
from ..utils import import_installed
pywbem = import_installed('pywbem')
from pywbem import Statistics  # noqa: E402
# pylint: enable=wrong-import-position, wrong-import-order, invalid-name


@pytest.mark.xfail(
    reason="Statistics and OperationStatistic have reference cycle")
@yagot.garbage_checked()
def test_leaks_Statistics_minimal():
    """
    Test function with a minimal Statistics object.
    """
    _ = Statistics()



#!/usr/bin/env python

"""
Test _warnings module.
"""

from __future__ import absolute_import, print_function

import six
import pytest

# pylint: disable=wrong-import-position, wrong-import-order, invalid-name
# pylint: disable=redefined-builtin
from ...utils import import_installed
pywbem = import_installed('pywbem')
from pywbem import Warning, ToleratedServerIssueWarning  # noqa: E402
# pylint: enable=wrong-import-position, wrong-import-order, invalid-name
# pylint: enable=redefined-builtin


# Test connection ID used for showing connection information in exception
# messages
TEST_CONN_ID = 'fake-conn-id'

# The expected connection information in exception messages
TEST_CONN_STR = 'Connection id: %s' % TEST_CONN_ID
TEST_CONN_STR_NONE = 'Connection id: None'


def _assert_subscription(exc):
    """
    Test the exception defined by exc for required args.
    """

    # Access by subscription is only supported in Python 2:
    if six.PY2:
        assert exc[:] == exc.args
        for i, _ in enumerate(exc.args):
            assert exc[i] == exc.args[i]
            assert exc[i:] == exc.args[i:]
            assert exc[0:i] == exc.args[0:i]
    else:
        try:
            _ = exc[:]
        except TypeError:
            pass
        else:
            assert False, "Access by slice did not fail in Python 3"
        for i, _ in enumerate(exc.args):
            try:
                _ = exc[i]  # noqa: F841
            except TypeError:
                pass
            else:
                assert False, "Access by index did not fail in Python 3"


def _assert_connection(exc, conn_id_kwarg, exp_conn_str):
    """
    Test the exception defined by exc for connection related information.
    """
    exp_conn_id = conn_id_kwarg.get('conn_id', None)
    assert exc.conn_id is exp_conn_id
    assert exc.conn_str == exp_conn_str


@pytest.fixture(params=[
    # The warning classes for which the simple test should be done:
    Warning,
    ToleratedServerIssueWarning,
], scope='module')
def simple_class(request):
    """
    Fixture representing variations of the simple warning classes.

    Returns the warning class.
    """
    return request.param


@pytest.fixture(params=[
    # Tuple of positional init arguments for the simple warning classes
    (),
    ('',),
    ('foo',),
    ('foo', 42),
], scope='module')
def simple_args(request):
    """
    Fixture representing variations of positional init arguments for the simple
    warning classes.

    Returns a tuple of positional arguments for initializing a warning object.
    """
    return request.param


@pytest.fixture(params=[
    # Tuple of (conn_id_kwarg, exp_conn_str)
    (dict(), TEST_CONN_STR_NONE),
    (dict(conn_id=None), TEST_CONN_STR_NONE),
    (dict(conn_id=TEST_CONN_ID), TEST_CONN_STR),
], scope='module')
def conn_info(request):
    """
    Fixture representing variations for the conn_id keyword argument for all
    warning classes, and the corresponding expected connection info string.

    Returns a tuple of:
    * conn_id_kwarg: dict with the 'conn_id' keyword argument. May be empty.
    * exp_conn_str: Expected connection info string.
    """
    return request.param


def test_simple(simple_class, simple_args, conn_info):
    # pylint: disable=redefined-outer-name
    """
    Test the simple warning classes.

    Note, the Python `Warning` class is derived from the `Exception` class and
    thus can be treated like exception classes.
    """

    conn_id_kwarg, exp_conn_str = conn_info

    exc = simple_class(*simple_args, **conn_id_kwarg)

    # exc has no len()
    assert len(exc.args) == len(simple_args)
    for i, _ in enumerate(simple_args):
        assert exc.args[i] == simple_args[i]
        assert exc.args[i:] == simple_args[i:]
        assert exc.args[0:i] == simple_args[0:i]
        assert exc.args[:] == simple_args[:]

    _assert_connection(exc, conn_id_kwarg, exp_conn_str)
    _assert_subscription(exc)
